import os
import sys

from gibson.api.Cli import Cli
from gibson.command.BaseCommand import BaseCommand


class Question(BaseCommand):
    TOKEN_FILE = "file://"
    TOKEN_PYTHON = "py://"
    TOKEN_SQL = "sql://"

    def execute(self):
        if len(sys.argv) < 3:
            self.usage()

        instructions = ""
        has_file = False
        has_python = False
        has_sql = False
        for term in sys.argv[2:]:
            if self.TOKEN_FILE in term:
                token, path = term.split(self.TOKEN_FILE)

                try:
                    with open(path, "r") as f:
                        contents = f.read()
                except FileNotFoundError:
                    self.__file_not_found(path)

                has_file = True
                instructions += f"\n\n{contents}\n\n"
            elif self.TOKEN_PYTHON in term:
                token, import_ = term.split(self.TOKEN_PYTHON)
                import_ = import_.replace(".", "/")

                contents = None
                for path in os.environ["PYTHONPATH"].split(":"):
                    try:
                        with open(f"{path}/{import_}.py", "r") as f:
                            contents = f.read()
                    except FileNotFoundError:
                        pass

                if contents is None:
                    self.__python_import_not_found(import_.replace("/", "."))

                has_python = True
                instructions += f"\n\n{contents}\n\n"
            elif self.TOKEN_SQL in term:
                token, name = term.split(self.TOKEN_SQL)

                entity = self.memory.recall_entity(name)
                if entity is None:
                    self.conversation.not_sure_no_entity(
                        self.configuration.project.name, name
                    )
                    exit(1)

                has_sql = True
                instructions += f"\n\n{entity['definition']}\n\n"
            else:
                instructions += f"{term} "

        if instructions != "":
            cli = Cli(self.configuration)
            response = cli.llm_query(instructions, has_file, has_python, has_sql)

        if response["entities"] is not None and len(response["entities"]) > 0:
            self.memory.remember_last({"entities": response["entities"]})

        self.conversation.raw_llm_response()
        print(response["commentary"].rstrip())

        if response["entities"] is not None and len(response["entities"]) > 0:
            self.conversation.newline()
            self.conversation.entities_hijacked()
            self.conversation.newline()

        self.conversation.newline()

    def __file_not_found(self, path):
        self.conversation.display_project(self.configuration.project.name)
        self.conversation.type(f'404, My Friend. Cannot find file "{path}".\n')
        self.conversation.newline()
        exit(1)

    def __python_import_not_found(self, import_):
        self.conversation.display_project(self.configuration.project.name)
        self.conversation.type(f'That\'s a misfire, "{import_}" does not exist.\n')
        self.conversation.newline()
        exit(1)

    def usage(self):
        self.conversation.display_project(self.configuration.project.name)
        self.conversation.type(
            f"usage: {self.configuration.command} ? [instructions]\n"
            + "  where [instructions] is natural language\n"
            + f"    use {self.TOKEN_FILE}[path] to import a file from the filesystem\n"
            + f"    use {self.TOKEN_PYTHON}[import] to import a file from PYTHONPATH\n"
            + f"    use {self.TOKEN_SQL}[entity name] to import the SQL\n"
        )
        self.conversation.newline()
        exit(1)
